part of '../flutterx_live_data.dart';

class MutableLiveData<T> extends LiveData<T> {
  MutableLiveData.late();

  MutableLiveData({required T initialValue}) {
    setData(initialValue);
  }

  set value(T value) => setData(value);
}

class MutableMediatorLiveData<T> extends MutableLiveData<T> with MediatorMixin<T> {
  MutableMediatorLiveData.late() : super.late();

  MutableMediatorLiveData({required T initialValue}) : super(initialValue: initialValue);
}

class HistoryMutableLiveData<T> extends MutableLiveData<T> with HistoryMixin<T> {
  HistoryMutableLiveData.late() : super.late();

  HistoryMutableLiveData({required T initialValue}) : super(initialValue: initialValue);
}

class StorageMutableLiveData<T> extends MutableLiveData<T> with StorageMixin<T> {
  @override
  final String key;

  @override
  final Map<String, String> _storage;

  @override
  final T _defaultValue;

  @override
  final String Function(T value) _encode;

  StorageMutableLiveData({
    required this.key,
    required Map<String, String> storage,
    required T defaultValue,
    String Function(T value) encode = jsonEncode,
    required T Function(String data) decode,
  })  : _storage = storage,
        _encode = encode,
        _defaultValue = defaultValue,
        super(initialValue: StorageMixin.resolve(key, storage, defaultValue, decode));
}
